import requests
from licensespring.api.authorization import autorization_headers

from licensespring.api.error import ClientError
from licensespring.api.hardware import HardwareIdProvider
from licensespring.api.signature import verify_license_signature


class APIClient:
    def __init__(
        self,
        api_key,
        shared_key,
        hardware_id_provider=HardwareIdProvider,
        api_domain="api.licensespring.com",
        api_version="v4",
        verify_license_signature=True,
    ):
        self.api_key = api_key
        self.shared_key = shared_key

        self.hardware_id_provider = hardware_id_provider()

        self.api_base = "https://{}/api/{}".format(api_domain, api_version)

        self.verify_license_signature = verify_license_signature

    def api_url(self, endpoint):
        return "{}{}".format(self.api_base, endpoint)

    def request_headers(self, custom_headers={}):
        headers = {"Content-Type": "application/json"}
        authorization_headers = autorization_headers(self.api_key, self.shared_key)
        return {**headers, **authorization_headers, **custom_headers}

    def request_generic_data(
        self, product, hardware_id=None, license_key=None, username=None, password=None
    ):
        data = {
            "product": product,
            "hardware_id": hardware_id
            if hardware_id
            else self.hardware_id_provider.get_id(),
        }
        if license_key:
            data["license_key"] = license_key
        if username:
            data["username"] = username
        if password:
            data["password"] = password

        return data

    def request_additional_data(self, data, additional_data):
        for key, value in additional_data.items():
            if key is not None:
                data["key"] = value
        return data

    def send_request(self, method, endpoint, params=None, data=None):
        response = requests.request(
            method=method,
            url=self.api_url(endpoint),
            headers=self.request_headers(),
            params=params,
            json=data,
        )
        if 400 <= response.status_code < 500:
            raise ClientError(response)
        else:
            response.raise_for_status()
        return response

    def activate_license(
        self, product, hardware_id=None, license_key=None, username=None, password=None
    ):
        data = self.request_generic_data(
            product, hardware_id, license_key, username, password
        )

        response = self.send_request(
            method="post",
            endpoint="/activate_license",
            data=data,
        )
        response_json = response.json()

        if self.verify_license_signature:
            verify_license_signature(
                hardware_id,
                license_key,
                response_json.get("validity_period"),
                response_json.get("license_signature"),
            )

        return response_json

    def deactivate_license(
        self, product, hardware_id=None, license_key=None, username=None, password=None
    ):
        data = self.request_generic_data(
            product, hardware_id, license_key, username, password
        )

        response = self.send_request(
            method="post",
            endpoint="/deactivate_license",
            data=data,
        )
        return True if response.status_code == 200 else False

    def check_license(self, product, hardware_id=None, license_key=None, username=None):
        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )
        response = self.send_request(
            method="get",
            endpoint="/check_license",
            params=data,
        )
        response_json = response.json()

        if self.verify_license_signature:
            verify_license_signature(
                hardware_id,
                license_key,
                response_json.get("validity_period"),
                response_json.get("license_signature"),
            )

        return response_json

    # TODO activate_offline

    # TODO deactivate_offline

    def add_consumption(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        password=None,
        consumptions=None,
        max_overages=None,
        allow_overages=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, password
        )
        data = self.request_additional_data(
            data=data,
            additional_data={
                "consumptions": consumptions,
                "max_overages": max_overages,
                "allow_overages": allow_overages,
            },
        )

        response = self.send_request(
            method="post",
            endpoint="/add_consumption",
            json=data,
        )

        return response.json()

    def add_feature_consumption(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        password=None,
        feature=None,
        consumptions=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, password
        )
        data["feature"] = feature
        data = self.request_additional_data(
            data=data,
            additional_data={
                "consumptions": consumptions,
            },
        )

        response = self.send_request(
            method="post",
            endpoint="/add_feature_consumption",
            json=data,
        )

        return response.json()

    def trial_key(
        self,
        product,
        hardware_id=None,
        email=None,
        license_policy=None,
        first_name=None,
        last_name=None,
        phone=None,
        address=None,
        postcode=None,
        state=None,
        country=None,
        city=None,
        reference=None,
    ):

        data = self.request_generic_data(product, hardware_id, None, None, None)
        data = self.request_additional_data(
            data=data,
            additional_data={
                "email": email,
                "license_policy": license_policy,
                "first_name": first_name,
                "last_name": last_name,
                "phone": phone,
                "address": address,
                "postcode": postcode,
                "state": state,
                "country": country,
                "city": city,
                "reference": reference,
            },
        )

        response = self.send_request(
            method="get",
            endpoint="/trial_key",
            json=data,
        )

        return response.json()

    def product_details(
        self,
        product,
    ):
        data = {"product": product}

        response = self.send_request(
            method="get",
            endpoint="/product_details",
            json=data,
        )

        return response.json()

    def track_device_variables(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        variables=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )
        data["variables"] = variables

        response = self.send_request(
            method="post",
            endpoint="/track_device_variables",
            json=data,
        )

        return True if response.status_code == 200 else False

    def floating_release(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )

        response = self.send_request(
            method="post",
            endpoint="/floating/release",
            json=data,
        )

        return True if response.status_code == 200 else False

    def floating_borrow(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        password=None,
        borrowed_until=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, password
        )
        data["borrowed_until"] = borrowed_until

        response = self.send_request(
            method="post",
            endpoint="/floating/borrow",
            json=data,
        )

        return response.json()

    def change_password(self, username, password, new_password):
        data = {
            "username": username,
            "password": password,
            "new_password": new_password,
        }

        response = self.send_request(
            method="post",
            endpoint="/change_password",
            json=data,
        )

        return True if response.status_code == 200 else False

    def versions(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        env=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )
        data = self.request_additional_data(
            data=data,
            additional_data={
                "env": env,
            },
        )

        response = self.send_request(
            method="get",
            endpoint="/versions",
            json=data,
        )

        return response.json()

    def installation_file(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
        env=None,
        version=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )
        data = self.request_additional_data(
            data=data,
            additional_data={
                "env": env,
                "version": version,
            },
        )

        response = self.send_request(
            method="get",
            endpoint="/installation_file",
            params=data,
        )

        return response.json()

    def customer_license_users(self, product, customer):
        data = {
            "product": product,
            "customer": customer,
        }

        response = self.send_request(
            method="get",
            endpoint="/customer_license_users",
            params=data,
        )

        return response.json()

    def sso_url(self, product, customer_account_code):
        data = {
            "product": product,
            "customer_account_code": customer_account_code,
        }

        response = self.send_request(
            method="get",
            endpoint="/sso_url",
            params=data,
        )

        return response.json()

    def get_device_variables(
        self,
        product,
        hardware_id=None,
        license_key=None,
        username=None,
    ):

        data = self.request_generic_data(
            product, hardware_id, license_key, username, None
        )

        response = self.send_request(
            method="get",
            endpoint="/get_device_variables",
            params=data,
        )

        return response.json()



import base64

from Crypto.Hash import SHA256
from Crypto.PublicKey import RSA
from Crypto.Signature.pkcs1_15 import PKCS115_SigScheme

PUBLIC_KEY = """-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAwEt7ZQWSzeBYNBwyi3KW
+XU/6I+ju90R5ZpTvSE4VWL8KSgVJ6bWKIhaKTL2hbUpIQQgS5ZKfa4SEEMdIm5k
xe0u2F64JFAVQunx/1O/UXsD7ADVt93Q/hxA9Npa16tKepZoyhi79sxpaxyy/WTd
sZKuTLApX7bWX6/XPhaPjNyiWeeK2Naka44B+F+PwN/ey3rZUra3pBltShwy2dmK
IxJVmprf5ttMYBB+ouqPin3VgDw5Jq1FZwpLBiOe+ogR+sHu7QYWwLq6AgC4e3Jq
gryEvqfJr/XkvOnAdcAIZm6tK6DdnLWCy6Onrk7t0VQK5nnGF1CKE+jcAByWopcI
VCwfYrs1UCSY1YibKXNJoyvEhgLyCC+KAgXKf8omcg1Q18XKu5XSrvrCzXLnRnaF
NpFDjcg5AJJab78hX7qGPC+e8PjuBYwh2vtx5mFj7/c+T59JM/vXwwvW9DsnDztD
WEDFhzGanU71NgwrZbRoNSyTW0UjbZsyJVSIX6233Ng0y5L9mDe8p8P+u1B0sXAA
ozEKL/yG4Qu5r4LIw4iv6JVPT1xbWlH4Vc4KaN3toaf4+G0EkFy6ncvBncWifLAR
SoWpQ5YsygDdQNVYdmMsoQ76UTuNxo3eZ0sQJjbQVBqlkcXrAdtUCpuojPRsl/Xs
9zVOJrzZPG0I98E5quNbRkMCAwEAAQ==
-----END PUBLIC KEY-----"""


def verify_signature(message, signature):
    hash = SHA256.new(message)
    rsa_public_key = RSA.importKey(PUBLIC_KEY)
    verifier = PKCS115_SigScheme(rsa_public_key)
    verifier.verify(hash, signature)


def verify_license_signature(hardware_id, license_key, validity_period, signature_b64):
    message = (
        "{}#{}#{}".format(
            hardware_id, license_key, validity_period if validity_period else ""
        )
        .lower()
        .encode()
    )
    signature = base64.b64decode(signature_b64)
    verify_signature(message, signature)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['licensespring', 'licensespring.api']

package_data = \
{'': ['*']}

install_requires = \
['pycryptodome>=3.14.1,<4.0.0', 'requests>=2.27.1,<3.0.0']

setup_kwargs = {
    'name': 'licensespring',
    'version': '0.2.0',
    'description': 'LicenseSpring Python Library',
    'long_description': '# LicenseSpring Python Library\n\nThe LicenseSpring Python Library provides convenient access to the LicenseSpring API from\napplications written in the Python language.\n\n\n## Initialization\n\n### Poetry\n\nThis project uses [Poetry](https://python-poetry.org/) for packaging and dependency management.\nFor installation and usage see https://python-poetry.org/docs/.\n\nConfigure Poetry to create virtualenv inside the project’s root directory: \n```\npoetry config virtualenvs.in-project true\n```\n\nInstall dependencies: \n```\npoetry install\n```\n\n\n## Testing\n\nThis project uses [pytest](https://docs.pytest.org/en/7.1.x/) framework for testing.\n\nRun tests: \n```\npoetry run pytest\n```\n\n\n## Formatting\n\nThis project uses [black](https://github.com/psf/black) for code formatting.\n\nFormat code:\n```\npoetry run black .\n```\n\n\n## Building and Publishing\n\nThis project is published at [Python Package Index](https://pypi.org/project/licensespring/).\n\nThe PyPI token must be configured in Poetry for publishing:\n```\npoetry config pypi-token.pypi {token}\n```\n\nDefine the new version before building and publishing:\n```\npoetry version {version}\n```\n\nBuild the source and wheels archives:\n```\npoetry build\n```\n\nPublishes the package (previously built with the build command):\n```\npoetry publish\n```\n\nA single command for both building and publishing is also possible:\n```\npoetry publish --build\n```\n\n\n## Installation\n\nInstall `licensespring` library:\n\n```\npip install licensespring\n```\n\n## Hardware (Device) IDs\n\nThis library provides a preconfigured identity provider which uses [uuid.getnode()](https://docs.python.org/3/library/uuid.html#uuid.getnode) to generate unique ID per device as described:\n\n> Get the hardware address as a 48-bit positive integer. The first time this runs, it may launch a separate program, which could be quite slow. If all attempts to obtain the hardware address fail, we choose a random 48-bit number with the multicast bit (least significant bit of the first octet) set to 1 as recommended in RFC 4122. “Hardware address” means the MAC address of a network interface. On a machine with multiple network interfaces, universally administered MAC addresses (i.e. where the second least significant bit of the first octet is unset) will be preferred over locally administered MAC addresses, but with no other ordering guarantees.\n\n## Usage Examples\n\n### Check license\n```python\nfrom licensespring.api import APIClient\n\napi_client = APIClient(api_key="_your_api_key_", shared_key="_your_shared_key_")\n\nlicense_data = api_client.check_license("_your_hardware_id_", "_your_license_key_", "_your_product_code_")\n\nprint(license_data)\n```\n',
    'author': 'Toni Sredanović',
    'author_email': 'toni@licensespring.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://licensespring.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
